@@ -2,9 +2,9 @@
  * 
  * hmm_f2.c
  * 
- * copyright (c) 2001, Karl W Broman, Johns Hopkins University
+ * copyright (c) 2001, 2002, Karl W Broman, Johns Hopkins University
  *
- * last modified Nov, 2001
+ * last modified Mar, 2002
  * first written Feb, 2001
  *
  * Licensed under the GNU General Public License version 2 (June, 1991)
@@ -53,7 +53,7 @@ double emit_f2(int obs_gen, int true_gen, double error_prob)
     if(true_gen != 1) return(log(1.0-error_prob/2.0));
     else return(log(error_prob/2.0));
   }
-  return(log(-1.0)); /* shouldn't get here */
+  return(0.0); /* shouldn't get here */
 }
     
   
@@ -121,7 +121,7 @@ double emit_f2b(int obs_gen, int true_gen, double error_prob)
     if(true_gen != 1) return(log(1.0-error_prob/2.0));
     else return(log(error_prob/2.0));
   }
-  return(log(-1.0)); /* shouldn't get here */
+  return(0.0); /* shouldn't get here */
 }
     
   
